@@ -18,6 +18,7 @@ py_include_dirs = [
     "/Library/Frameworks/Python.framework/Versions/%s/Headers" % version_string
 ]
 
+osx_version = os.popen('sw_vers | grep ProductVersion').read().split()[1]
 # MACOSX_DEPLOYMENT_TARGET can be set to 10.3 in most cases.
 # But for the built-in Python 2.5.1 on Leopard, it needs to be set for 10.5.
 # This looks like a bug that will be fixed in 2.5.2.  If Apple updates their
@@ -28,12 +29,14 @@ leopard_python_prefix = '/System/Library/Frameworks/Python.framework/Versions/2.
 full_version = "%s.%s.%s" % sys.version_info[:3]
 if python_prefix == leopard_python_prefix and full_version == '2.5.1':
     os.environ["MACOSX_DEPLOYMENT_TARGET"] = "10.5"
+elif osx_version >= "10.6":
+    os.environ["MACOSX_DEPLOYMENT_TARGET"] = "10.4"
 else:
     os.environ["MACOSX_DEPLOYMENT_TARGET"] = "10.3"
 
 compilers = ["gcc", "g++"]
 compiler_options = \
-    "-g -c -fno-strict-aliasing -Wno-long-double -no-cpp-precomp " \
+    "-g -c -fno-strict-aliasing -no-cpp-precomp " \
     "-mno-fused-madd -fno-common -dynamic " \
     .split()
 if gcc_pendantic:
@@ -51,8 +54,8 @@ linker_options = \
     "-Wl,-F.,-w -bundle -undefined dynamic_lookup" \
     .split()
 #linker_options = \
-#	"-Wl,-F.,-w -bundle -framework Python" \
-#	.split()
+#    "-Wl,-F.,-w -bundle -framework Python" \
+#    .split()
 
 class CCompilerError(PyrexError):
     pass
